'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _initStorage = require('./initStorage');

var _initStorage2 = _interopRequireDefault(_initStorage);

var _validateKey = require('./utils/validateKey');

var _validateKey2 = _interopRequireDefault(_validateKey);

var _getIndexedDB = require('./utils/getIndexedDB');

var _getIndexedDB2 = _interopRequireDefault(_getIndexedDB);

var _destroyDatabase = require('./utils/destroyDatabase');

var _destroyDatabase2 = _interopRequireDefault(_destroyDatabase);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var indexedDB = (0, _getIndexedDB2.default)();

function createInstance(_ref) {
  var storeName = _ref.storeName,
      _ref$version = _ref.version,
      version = _ref$version === undefined ? _constants.DEFAULT_VERSION : _ref$version,
      _ref$dbName = _ref.dbName,
      dbName = _ref$dbName === undefined ? _constants.DEFAULT_DB_NAME : _ref$dbName,
      _ref$options = _ref.options,
      options = _ref$options === undefined ? {} : _ref$options;

  if (!indexedDB) {
    throw new Error('No indexedDB implementation found!');
  }

  if (!storeName) {
    throw new Error('No store name provided for Indexed DB instance.');
  }

  if (!dbName) {
    throw new Error('No database name provided for Indexed DB instance.');
  }

  var databaseReadyPromise = (0, _initStorage2.default)({ dbName: dbName, storeName: storeName, version: version, options: options });

  // Retrieve an item from the desired object store under the provided key
  function getItem(name, key) {
    key = (0, _validateKey2.default)(key);

    return databaseReadyPromise.then(function (dbInstance) {
      return new Promise(function (resolve, reject) {
        var transaction = dbInstance.transaction(name, _constants.READ_ONLY);
        var objectStore = transaction.objectStore(name);

        var req = objectStore.get(key);

        req.onsuccess = function () {
          return resolve(req.result);
        };
        req.onabort = function () {
          return reject(transaction);
        };
        req.onerror = function () {
          return reject(transaction);
        };
      });
    });
  }

  // Store an item in the provided store name under the provided key
  function setItem(name, key, value) {
    key = (0, _validateKey2.default)(key);

    return databaseReadyPromise.then(function (dbInstance) {
      return new Promise(function (resolve, reject) {
        var transaction = dbInstance.transaction(name, _constants.READ_WRITE);
        var objectStore = transaction.objectStore(name);

        // Saving as `null` will result in a subsequent getItem returning `undefined`
        // Make it `undefined` immediately so that there is consistency
        if (value === null) {
          value = undefined;
        }

        objectStore.put(value, key);

        transaction.oncomplete = function () {
          return resolve(value, key);
        };
        transaction.onabort = function () {
          return reject(transaction);
        };
        transaction.onerror = function () {
          return reject(transaction);
        };
      });
    });
  }

  // Return all the items stored in the provided store name
  function getCollection(name) {
    return databaseReadyPromise.then(function (dbInstance) {
      return new Promise(function (resolve, reject) {
        var transaction = dbInstance.transaction(name, _constants.READ_WRITE);
        var objectStore = transaction.objectStore(name);

        var req = objectStore.getAll();

        req.onsuccess = function () {
          return resolve(req.result);
        };
        req.onabort = function () {
          return reject(req);
        };
        req.onerror = function () {
          return reject(req);
        };
      });
    });
  }

  // Remove an item from the provided store name under the provided key
  function removeItem(name, key) {
    key = (0, _validateKey2.default)(key);

    return databaseReadyPromise.then(function (dbInstance) {
      return new Promise(function (resolve, reject) {
        var transaction = dbInstance.transaction(name, _constants.READ_WRITE);
        var objectStore = transaction.objectStore(name);

        objectStore.delete(key);

        transaction.oncomplete = function () {
          return resolve();
        };
        transaction.onabort = function () {
          return reject(transaction);
        };
        transaction.onerror = function () {
          return reject(transaction);
        };
      });
    });
  }

  function clear(name) {
    return databaseReadyPromise.then(function (dbInstance) {
      return new Promise(function (resolve, reject) {
        try {
          var transaction = dbInstance.transaction(name, 'readwrite');
          var objectStore = transaction.objectStore(name);

          var req = objectStore.clear();

          var onError = function onError() {
            var err = req.error ? req.error : req.transaction.error;
            reject(err);
          };

          transaction.oncomplete = function () {
            return resolve();
          };
          transaction.onabort = onError;
          transaction.onerror = onError;
        } catch (err) {
          reject({
            storeName: name,
            error: err
          });
        }
      });
    });
  }

  // Get the size of the provided store name
  function length(name) {
    return databaseReadyPromise.then(function (dbInstance) {
      return new Promise(function (resolve, reject) {
        var transaction = dbInstance.transaction(name, _constants.READ_ONLY);
        var objectStore = transaction.objectStore(name);

        var req = objectStore.count();

        req.onsuccess = function () {
          return resolve(req.result);
        };
        req.onabort = function () {
          return reject(req);
        };
        req.onerror = function () {
          return reject(req);
        };
      });
    });
  }

  // Return all of the keys in the object store
  function keys(name) {
    return databaseReadyPromise.then(function (dbInstance) {
      return new Promise(function (resolve, reject) {
        var transaction = dbInstance.transaction(name, _constants.READ_ONLY);
        var objectStore = transaction.objectStore(name);

        try {
          var req = objectStore.openCursor();
          var storeKeys = [];

          req.onsuccess = function () {
            var cursor = req.result;

            // If the cursor is empty, send back the keys
            if (!cursor) {
              resolve(storeKeys);
              return;
            }

            // Otherwise keep going
            storeKeys.push(cursor.key);
            cursor.continue();
          };

          req.onerror = function () {
            return reject(req.error);
          };
        } catch (e) {
          reject(e);
        }
      });
    });
  }

  function iterate(name, callback) {
    return databaseReadyPromise.then(function (dbInstance) {
      return new Promise(function (resolve, reject) {
        var transaction = dbInstance.transaction(name, _constants.READ_ONLY);
        var objectStore = transaction.objectStore(name);

        try {
          var req = objectStore.openCursor();

          req.onsuccess = function () {
            var cursor = req.result;

            // If the cursor is empty, send back the keys
            if (!cursor) {
              resolve();
              return;
            }

            // Otherwise keep going
            callback(cursor.value, cursor.key, cursor);
            cursor.continue();
          };

          req.onerror = function () {
            return reject(req.error);
          };
        } catch (e) {
          reject(e);
        }
      });
    });
  }

  // Kills the connection to the DB
  function close() {
    return databaseReadyPromise.then(function (dbInstance) {
      return new Promise(function (resolve) {
        dbInstance.close();

        if (options.enableLogging) {
          console.info('Closed connection to: ', dbName);
        }

        resolve();
      });
    });
  }

  // Destroys the database instance
  function destroy() {
    return databaseReadyPromise.then(function (dbInstance) {
      dbInstance.close();

      return (0, _destroyDatabase2.default)(dbName, options);
    });
  }

  return {
    isReady: databaseReadyPromise,
    getItem: _lodash2.default.partial(getItem, storeName),
    setItem: _lodash2.default.partial(setItem, storeName),
    getCollection: _lodash2.default.partial(getCollection, storeName),
    removeItem: _lodash2.default.partial(removeItem, storeName),
    clear: _lodash2.default.partial(clear, storeName),
    length: _lodash2.default.partial(length, storeName),
    keys: _lodash2.default.partial(keys, storeName),
    iterate: _lodash2.default.partial(iterate, storeName),
    close: close,
    destroy: destroy
  };
}

exports.default = createInstance;